@@ -21,7 +21,7 @@ import datafusion
 from datafusion import (
     AggregateUDF,
     DataFrame,
-    ExecutionContext,
+    SessionContext,
     Expression,
     ScalarUDF,
     functions,
@@ -38,7 +38,7 @@ def test_datafusion_python_version():
 
 def test_class_module_is_datafusion():
     for klass in [
-        ExecutionContext,
+        SessionContext,
         Expression,
         DataFrame,
         ScalarUDF,
@@ -59,7 +59,7 @@ def test_import_from_functions_submodule():
 
 
 def test_classes_are_inheritable():
-    class MyExecContext(ExecutionContext):
+    class MyExecContext(SessionContext):
         pass
 
     class MyExpression(Expression):
