@@ -15,19 +15,34 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#[cfg(feature = "mimalloc")]
 use mimalloc::MiMalloc;
 use pyo3::prelude::*;
 
+#[allow(clippy::borrow_deref_ref)]
 pub mod catalog;
+#[allow(clippy::borrow_deref_ref)]
+mod config;
+#[allow(clippy::borrow_deref_ref)]
 mod context;
+#[allow(clippy::borrow_deref_ref)]
 mod dataframe;
+mod dataset;
+mod dataset_exec;
 pub mod errors;
+#[allow(clippy::borrow_deref_ref)]
 mod expression;
+#[allow(clippy::borrow_deref_ref)]
 mod functions;
+mod pyarrow_filter_expression;
+pub mod store;
+#[allow(clippy::borrow_deref_ref)]
 mod udaf;
+#[allow(clippy::borrow_deref_ref)]
 mod udf;
 pub mod utils;
 
+#[cfg(feature = "mimalloc")]
 #[global_allocator]
 static GLOBAL: MiMalloc = MiMalloc;
 
@@ -41,16 +56,21 @@ fn _internal(py: Python, m: &PyModule) -> PyResult<()> {
     m.add_class::<catalog::PyCatalog>()?;
     m.add_class::<catalog::PyDatabase>()?;
     m.add_class::<catalog::PyTable>()?;
-    m.add_class::<context::PyExecutionContext>()?;
+    m.add_class::<context::PySessionContext>()?;
     m.add_class::<dataframe::PyDataFrame>()?;
     m.add_class::<expression::PyExpr>()?;
     m.add_class::<udf::PyScalarUDF>()?;
     m.add_class::<udaf::PyAggregateUDF>()?;
+    m.add_class::<config::PyConfig>()?;
 
     // Register the functions as a submodule
     let funcs = PyModule::new(py, "functions")?;
     functions::init_module(funcs)?;
     m.add_submodule(funcs)?;
 
+    let store = PyModule::new(py, "object_store")?;
+    store::init_module(store)?;
+    m.add_submodule(store)?;
+
     Ok(())
 }
