@@ -19,15 +19,14 @@ use std::sync::Arc;
 
 use pyo3::{prelude::*, types::PyTuple};
 
-use datafusion::arrow::array::ArrayRef;
+use datafusion::arrow::array::{Array, ArrayRef};
 use datafusion::arrow::datatypes::DataType;
-use datafusion::arrow::pyarrow::PyArrowConvert;
+use datafusion::arrow::pyarrow::{PyArrowConvert, PyArrowType};
+use datafusion::common::ScalarValue;
 use datafusion::error::{DataFusionError, Result};
-use datafusion::logical_plan;
-use datafusion_common::ScalarValue;
-use datafusion_expr::Accumulator;
-use datafusion_expr::AccumulatorFunctionImplementation;
-use datafusion_expr::AggregateUDF;
+use datafusion_expr::{
+    create_udaf, Accumulator, AccumulatorFunctionImplementation, AggregateState, AggregateUDF,
+};
 
 use crate::expression::PyExpr;
 use crate::utils::parse_volatility;
@@ -44,9 +43,13 @@ impl RustAccumulator {
 }
 
 impl Accumulator for RustAccumulator {
-    fn state(&self) -> Result<Vec<ScalarValue>> {
-        Python::with_gil(|py| self.accum.as_ref(py).call_method0("state")?.extract())
-            .map_err(|e| DataFusionError::Execution(format!("{}", e)))
+    fn state(&self) -> Result<Vec<AggregateState>> {
+        let py_result: PyResult<Vec<ScalarValue>> =
+            Python::with_gil(|py| self.accum.as_ref(py).call_method0("state")?.extract());
+        match py_result {
+            Ok(r) => Ok(r.into_iter().map(AggregateState::Scalar).collect()),
+            Err(e) => Err(DataFusionError::Execution(format!("{}", e))),
+        }
     }
 
     fn evaluate(&self) -> Result<ScalarValue> {
@@ -79,6 +82,7 @@ impl Accumulator for RustAccumulator {
 
             // 1. cast states to Pyarrow array
             let state = state
+                .data()
                 .to_pyarrow(py)
                 .map_err(|e| DataFusionError::Execution(format!("{}", e)))?;
 
@@ -94,7 +98,7 @@ impl Accumulator for RustAccumulator {
 }
 
 pub fn to_rust_accumulator(accum: PyObject) -> AccumulatorFunctionImplementation {
-    Arc::new(move || -> Result<Box<dyn Accumulator>> {
+    Arc::new(move |_| -> Result<Box<dyn Accumulator>> {
         let accum = Python::with_gil(|py| {
             accum
                 .call0(py)
@@ -104,7 +108,7 @@ pub fn to_rust_accumulator(accum: PyObject) -> AccumulatorFunctionImplementation
     })
 }
 
-/// Represents a AggregateUDF
+/// Represents an AggregateUDF
 #[pyclass(name = "AggregateUDF", module = "datafusion", subclass)]
 #[derive(Debug, Clone)]
 pub struct PyAggregateUDF {
@@ -117,18 +121,18 @@ impl PyAggregateUDF {
     fn new(
         name: &str,
         accumulator: PyObject,
-        input_type: DataType,
-        return_type: DataType,
-        state_type: Vec<DataType>,
+        input_type: PyArrowType<DataType>,
+        return_type: PyArrowType<DataType>,
+        state_type: PyArrowType<Vec<DataType>>,
         volatility: &str,
     ) -> PyResult<Self> {
-        let function = logical_plan::create_udaf(
+        let function = create_udaf(
             name,
-            input_type,
-            Arc::new(return_type),
+            input_type.0,
+            Arc::new(return_type.0),
             parse_volatility(volatility)?,
             to_rust_accumulator(accumulator),
-            Arc::new(state_type),
+            Arc::new(state_type.0),
         );
         Ok(Self { function })
     }
