@@ -19,13 +19,14 @@ use std::sync::Arc;
 
 use pyo3::{prelude::*, types::PyTuple};
 
-use datafusion::arrow::array::ArrayRef;
+use datafusion::arrow::array::{make_array, Array, ArrayData, ArrayRef};
 use datafusion::arrow::datatypes::DataType;
-use datafusion::arrow::pyarrow::PyArrowConvert;
+use datafusion::arrow::pyarrow::{PyArrowConvert, PyArrowType};
 use datafusion::error::DataFusionError;
-use datafusion::logical_plan;
-use datafusion::physical_plan::functions::{make_scalar_function, ScalarFunctionImplementation};
+use datafusion::physical_plan::functions::make_scalar_function;
 use datafusion::physical_plan::udf::ScalarUDF;
+use datafusion_expr::create_udf;
+use datafusion_expr::function::ScalarFunctionImplementation;
 
 use crate::expression::PyExpr;
 use crate::utils::parse_volatility;
@@ -45,15 +46,14 @@ fn to_rust_function(func: PyObject) -> ScalarFunctionImplementation {
                 let py_args = PyTuple::new(py, py_args);
 
                 // 2. call function
-                let value = func.as_ref(py).call(py_args, None);
-                let value = match value {
-                    Ok(n) => Ok(n),
-                    Err(error) => Err(DataFusionError::Execution(format!("{:?}", error))),
-                }?;
+                let value = func
+                    .as_ref(py)
+                    .call(py_args, None)
+                    .map_err(|e| DataFusionError::Execution(format!("{:?}", e)))?;
 
                 // 3. cast to arrow::array::Array
-                let array = ArrayRef::from_pyarrow(value).unwrap();
-                Ok(array)
+                let array_data = ArrayData::from_pyarrow(value).unwrap();
+                Ok(make_array(array_data))
             })
         },
     )
@@ -72,14 +72,14 @@ impl PyScalarUDF {
     fn new(
         name: &str,
         func: PyObject,
-        input_types: Vec<DataType>,
-        return_type: DataType,
+        input_types: PyArrowType<Vec<DataType>>,
+        return_type: PyArrowType<DataType>,
         volatility: &str,
     ) -> PyResult<Self> {
-        let function = logical_plan::create_udf(
+        let function = create_udf(
             name,
-            input_types,
-            Arc::new(return_type),
+            input_types.0,
+            Arc::new(return_type.0),
             parse_volatility(volatility)?,
             to_rust_function(func),
         );
